@@ -34,7 +34,7 @@
  * Client to download via HTTP.
  * @see DownloadDialog
  */
-class DownloadClient : public HttpClient {
+class KID3_CORE_EXPORT DownloadClient : public HttpClient {
   Q_OBJECT
 public:
   /**
@@ -65,7 +65,7 @@ public:
    *
    * @return URL of image file, empty if no image URL found.
    */
-  static QUrl getImageUrl(const QString& url);
+  static QUrl getImageUrl(const QUrl& url);
 
 public slots:
   /**
