@@ -29,16 +29,20 @@
 #include "taggedfile.h"
 #include "frametablemodel.h"
 #include "iframeeditor.h"
+#include "pictureframe.h"
 
 /**
  * Constructor.
  *
+ * @param tagNr tag number
  * @param ftm frame table model
  * @param selModel item selection model
  */
-FrameList::FrameList(FrameTableModel* ftm, QItemSelectionModel* selModel) :
-  QObject(ftm), m_oldChangedFrames(0), m_taggedFile(0), m_frameTableModel(ftm),
-  m_selectionModel(selModel), m_cursorRow(-1), m_cursorColumn(-1)
+FrameList::FrameList(Frame::TagNumber tagNr,
+                     FrameTableModel* ftm, QItemSelectionModel* selModel) :
+  QObject(ftm), m_oldChangedFrames(0), m_taggedFile(0), m_frameEditor(0),
+  m_frameTableModel(ftm), m_selectionModel(selModel),
+  m_cursorRow(-1), m_cursorColumn(-1), m_tagNr(tagNr), m_addingFrame(false)
 {
   setObjectName(QLatin1String("FrameList"));
 }
@@ -113,8 +117,19 @@ QString FrameList::getSelectedName() const
  */
 bool FrameList::selectByName(const QString& name)
 {
-  int row = m_frameTableModel->getRowWithFrameName(name);
-  if (row < 0)
+  return selectByRow(m_frameTableModel->getRowWithFrameName(name));
+}
+
+/**
+ * Select a frame by row number in the frame table.
+ *
+ * @param row row of frame
+ *
+ * @return true if a frame could be selected.
+ */
+bool FrameList::selectByRow(int row)
+{
+  if (row < 0 || row >= m_frameTableModel->rowCount())
     return false;
 
   m_selectionModel->setCurrentIndex(m_frameTableModel->index(row, 0),
@@ -129,7 +144,7 @@ void FrameList::setModelFromTaggedFile()
 {
   if (m_taggedFile) {
     FrameCollection frames;
-    m_taggedFile->getAllFramesV2(frames);
+    m_taggedFile->getAllFrames(m_tagNr, frames);
     m_frameTableModel->transferFrames(frames);
   }
 }
@@ -144,7 +159,7 @@ bool FrameList::deleteFrame()
   saveCursor();
   Frame frame;
   if (getSelectedFrame(frame) && m_taggedFile) {
-    m_taggedFile->deleteFrameV2(frame);
+    m_taggedFile->deleteFrame(m_tagNr, frame);
     setModelFromTaggedFile();
     restoreCursor();
     return true;
@@ -153,63 +168,110 @@ bool FrameList::deleteFrame()
 }
 
 /**
- * Let the user select and edit a frame type and then edit the frame.
- * Add the frame if the edits are accepted.
- * frameEdited() is emitted with the added frame.
+ * Set editor for frames.
  *
  * @param frameEditor frame editor
  */
-void FrameList::selectAddAndEditFrame(IFrameEditor* frameEditor)
+void FrameList::setFrameEditor(IFrameEditor* frameEditor)
 {
-  if (m_taggedFile) {
-    Frame frame;
-    if (frameEditor->selectFrame(&frame, m_taggedFile)) {
-      m_frame = frame;
-      addAndEditFrame(frameEditor);
-      return;
+  if (m_frameEditor != frameEditor) {
+    if (m_frameEditor) {
+      QObject* obj = m_frameEditor->qobject();
+      disconnect(obj, SIGNAL(frameSelected(Frame::TagNumber,const Frame*)),
+                 this, SLOT(onFrameSelected(Frame::TagNumber,const Frame*)));
+      disconnect(obj, SIGNAL(frameEdited(Frame::TagNumber,const Frame*)),
+                 this, SLOT(onFrameEdited(Frame::TagNumber,const Frame*)));
+    }
+    m_frameEditor = frameEditor;
+    if (m_frameEditor) {
+      QObject* obj = m_frameEditor->qobject();
+      connect(obj, SIGNAL(frameSelected(Frame::TagNumber,const Frame*)),
+              this, SLOT(onFrameSelected(Frame::TagNumber,const Frame*)));
+      connect(obj, SIGNAL(frameEdited(Frame::TagNumber,const Frame*)),
+              this, SLOT(onFrameEdited(Frame::TagNumber,const Frame*)));
     }
   }
-  emit frameEdited(0);
+}
+
+/**
+ * Let the user select and edit a frame type and then edit the frame.
+ * Add the frame if the edits are accepted.
+ * frameEdited() is emitted with the added frame.
+ */
+void FrameList::selectAddAndEditFrame()
+{
+  if (m_taggedFile && m_frameEditor) {
+    m_addingFrame = true;
+    m_frameEditor->setTagNumber(m_tagNr);
+    m_frameEditor->selectFrame(&m_frame, m_taggedFile);
+  } else {
+    emit frameAdded(0);
+  }
+}
+
+/**
+ * Called when the frame is selected.
+ * @param tagNr tag number
+ * @param frame selected frame, 0 if none selected.
+ */
+void FrameList::onFrameSelected(Frame::TagNumber tagNr, const Frame* frame)
+{
+  if (tagNr != m_tagNr)
+    return;
+
+  if (frame) {
+    addAndEditFrame();
+  } else {
+    emit frameAdded(0);
+  }
 }
 
 /**
  * Add and edit a new frame.
  * frameEdited() is emitted with the added frame.
- *
- * @param frameEditor editor for frame fields
  */
-void FrameList::addAndEditFrame(IFrameEditor* frameEditor)
+void FrameList::addAndEditFrame()
 {
   if (m_taggedFile) {
-    m_oldChangedFrames = m_taggedFile->getChangedFramesV2();
-    if (!m_taggedFile->addFrameV2(m_frame)) {
-      emit frameEdited(0);
-    } else if (frameEditor) {
-      connect(frameEditor->frameEditedEmitter(),
-              SIGNAL(frameEdited(const Frame*)),
-              this, SLOT(onFrameEdited(const Frame*)), Qt::UniqueConnection);
-      frameEditor->editFrameOfTaggedFile(&m_frame, m_taggedFile);
+    m_oldChangedFrames = m_taggedFile->getChangedFrames(m_tagNr);
+    if (!m_taggedFile->addFrame(m_tagNr, m_frame)) {
+      emit frameAdded(0);
+    } else if (m_frameEditor) {
+      m_addingFrame = true;
+      m_frameEditor->setTagNumber(m_tagNr);
+      m_frameEditor->editFrameOfTaggedFile(&m_frame, m_taggedFile);
     } else {
-      onFrameEdited(&m_frame);
+      m_addingFrame = true;
+      onFrameEdited(m_tagNr, &m_frame);
     }
   } else {
-    emit frameEdited(0);
+    emit frameAdded(0);
+  }
+}
+
+/**
+ * Edit the current frame.
+ * The frame and its file have to be set using setFrame() and setTaggedFile().
+ */
+void FrameList::editFrame()
+{
+  if (m_frameEditor) {
+    m_addingFrame = false;
+    m_frameEditor->setTagNumber(m_tagNr);
+    m_frameEditor->editFrameOfTaggedFile(&m_frame, m_taggedFile);
   }
 }
 
 /**
  * Called when the frame is edited.
+ * @param tagNr tag number
  * @param frame edited frame, 0 if canceled
  */
-void FrameList::onFrameEdited(const Frame* frame)
+void FrameList::onFrameEdited(Frame::TagNumber tagNr, const Frame* frame)
 {
-  if (QObject* emitter = sender()) {
-    if (emitter->metaObject()->indexOfSignal("frameEdited(const Frame*)") != -1)
-    {
-      disconnect(emitter, SIGNAL(frameEdited(const Frame*)),
-                 this, SLOT(onFrameEdited(const Frame*)));
-    }
-  }
+  if (tagNr != m_tagNr)
+    return;
+
   if (frame) {
     int index = frame->getIndex();
     setModelFromTaggedFile();
@@ -217,10 +279,16 @@ void FrameList::onFrameEdited(const Frame* frame)
       setSelectedId(index);
     }
   } else {
-    m_taggedFile->deleteFrameV2(m_frame);
-    m_taggedFile->setChangedFramesV2(m_oldChangedFrames);
+    if (m_addingFrame) {
+      m_taggedFile->deleteFrame(m_tagNr, m_frame);
+      m_taggedFile->setChangedFrames(m_tagNr, m_oldChangedFrames);
+    }
+  }
+  if (m_addingFrame) {
+    emit frameAdded(frame);
+  } else {
+    emit frameEdited(frame);
   }
-  emit frameEdited(frame);
 }
 
 /**
@@ -230,13 +298,27 @@ void FrameList::onFrameEdited(const Frame* frame)
  */
 bool FrameList::pasteFrame() {
   if (m_taggedFile && m_frame.getType() != Frame::FT_UnknownFrame) {
-    m_taggedFile->addFrameV2(m_frame);
-    m_taggedFile->setFrameV2(m_frame);
+    m_taggedFile->addFrame(m_tagNr, m_frame);
+    m_taggedFile->setFrame(m_tagNr, m_frame);
     return true;
   }
   return false;
 }
 
+/**
+ * Add a suitable field list for the frame in the copy buffer if missing.
+ */
+void FrameList::addFrameFieldList()
+{
+  if (m_taggedFile) {
+    m_taggedFile->addFieldList(m_tagNr, m_frame);
+    if (m_frame.getFieldList().isEmpty() &&
+        m_frame.getType() == Frame::FT_Picture) {
+      PictureFrame::setFields(m_frame);
+    }
+  }
+}
+
 /**
  * Save the current cursor position.
  */
